/*
 *      lark-export-dialog.c
 *
 *      Copyright 2016-2018 Alex <alex@linuxonly.ru>
 *
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>

#include "lark-export-dialog.h"

enum {
	LARK_EXPORT_DIALOG_FORMAT_ID_COLUMN = 0,
	LARK_EXPORT_DIALOG_FORMAT_NAME_COLUMN,
	LARK_EXPORT_DIALOG_FORMAT_COLUMNS
};

enum {
	LARK_EXPORT_DIALOG_SPLIT_MODE_DISABLED = 0,
	LARK_EXPORT_DIALOG_SPLIT_MODE_SIZE,
	LARK_EXPORT_DIALOG_SPLIT_MODE_TIME
};

enum {
	PROP_0,
	PROP_GSTREAMER,
	PROP_DOCUMENT_LOADER,
	PROP_EXPORT_SETTINGS,
	LAST_PROP
};

static GParamSpec *properties [LAST_PROP];

enum {
	SIGNAL_EXPORT_STARTED,
	LAST_SIGNAL
};

static guint signals [LAST_SIGNAL];

struct _LarkExportDialog
{
	GtkDialog parent_instance;
	/*Objects*/
	LarkGstreamer *gstreamer;
	LarkDocumentLoader *docloader;
	GSettings *expsettings;
	/*Widgets*/
	GtkButton *LarkExportExportButton;
	GtkEntry *LarkExportFile;
	GtkFileChooserButton *LarkExportFolder;
	GtkComboBox *LarkExportFormat;
	GtkSpinButton *LarkExportQuality;
	GtkAdjustment *LarkExportQualityAdjustment;
	GtkComboBoxText *LarkExportSplitMode;
	GtkSpinButton *LarkExportSplitLimit;
	GtkAdjustment *LarkExportSplitLimitAdjustment;
	GtkEntry *LarkExportTitle;
	GtkEntry *LarkExportAuthor;
	GtkEntry *LarkExportComment;
};

struct _LarkExportDialogClass
{
	GtkDialogClass parent_class;
};

G_DEFINE_TYPE (LarkExportDialog, lark_export_dialog, GTK_TYPE_DIALOG)


static void lark_export_dialog_export_button_clicked(GtkButton *button, LarkExportDialog *self)
{
	gchar *folder;
	GtkTreeModel *model;
	GtkTreeIter iter;
	gint targetid;
		
	/*Filename*/
	lark_gstreamer_set_file_name(self->gstreamer, (gchar *)gtk_entry_get_text(self->LarkExportFile));
	/*Folder*/
	folder = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(self->LarkExportFolder));
	if (folder != NULL) {
		lark_gstreamer_set_path(self->gstreamer, folder);
		/*Save setting*/
		g_settings_set_string(self->expsettings, "export-directory", folder);
	}
	/*Format*/
	targetid = LARK_GSTREAMER_TARGET_WAV;
	model = gtk_combo_box_get_model(self->LarkExportFormat);
	if (model != NULL) {
		if (gtk_combo_box_get_active_iter(self->LarkExportFormat, &iter)) {
			gtk_tree_model_get(model, &iter, LARK_EXPORT_DIALOG_FORMAT_ID_COLUMN, &targetid, -1);
			lark_gstreamer_set_target(self->gstreamer, targetid);
			/*Save setting*/
			switch (targetid) {
				case LARK_GSTREAMER_TARGET_WAV:
					g_settings_set_string(self->expsettings, "file-format", "wav");
					break;
				case LARK_GSTREAMER_TARGET_VORBIS:
					g_settings_set_string(self->expsettings, "file-format", "ogg-vorbis");
					break;
				case LARK_GSTREAMER_TARGET_OPUS:
					g_settings_set_string(self->expsettings, "file-format", "ogg-opus");
					break;
				case LARK_GSTREAMER_TARGET_FLAC:
					g_settings_set_string(self->expsettings, "file-format", "flac");
					break;
				case LARK_GSTREAMER_TARGET_MP3:
					g_settings_set_string(self->expsettings, "file-format", "mp3");
					break;
				case LARK_GSTREAMER_TARGET_AAC:
					g_settings_set_string(self->expsettings, "file-format", "aac");
					break;
				default:
					g_settings_set_string(self->expsettings, "file-format", "wav");
					break;
			}
		}
	}
	/*Quality*/
	lark_gstreamer_set_quality(self->gstreamer, (guint)gtk_adjustment_get_value(self->LarkExportQualityAdjustment));
	/*Save setting*/
	g_settings_set_uint(self->expsettings, "encoding-quality", (guint)gtk_adjustment_get_value(self->LarkExportQualityAdjustment));
	
	/*Splitting*/
	switch (gtk_combo_box_get_active(GTK_COMBO_BOX(self->LarkExportSplitMode))) {
		case LARK_EXPORT_DIALOG_SPLIT_MODE_SIZE:
			lark_gstreamer_set_limit_mode(self->gstreamer, LARK_GSTREAMER_LIMIT_MODE_SIZE);
			g_settings_set_string(self->expsettings, "split-mode", "size");
			lark_gstreamer_set_size_limit(self->gstreamer, (guint)gtk_adjustment_get_value(self->LarkExportSplitLimitAdjustment));
			g_settings_set_uint(self->expsettings, "size-limit", (guint)gtk_adjustment_get_value(self->LarkExportSplitLimitAdjustment));
			break;
		case LARK_EXPORT_DIALOG_SPLIT_MODE_TIME:
			lark_gstreamer_set_limit_mode(self->gstreamer, LARK_GSTREAMER_LIMIT_MODE_TIME);
			g_settings_set_string(self->expsettings, "split-mode", "time");
			lark_gstreamer_set_time_limit(self->gstreamer, (guint)gtk_adjustment_get_value(self->LarkExportSplitLimitAdjustment));
			g_settings_set_uint(self->expsettings, "time-limit", (guint)gtk_adjustment_get_value(self->LarkExportSplitLimitAdjustment));
			break;
		default:
			lark_gstreamer_set_limit_mode(self->gstreamer, LARK_GSTREAMER_LIMIT_MODE_NONE);
			g_settings_set_string(self->expsettings, "split-mode", "disabled");
			break;
	}
	
	/*Metatags*/
	lark_gstreamer_set_title_tag(self->gstreamer, (gchar *)gtk_entry_get_text(self->LarkExportTitle));
	lark_gstreamer_set_author_tag(self->gstreamer, (gchar *)gtk_entry_get_text(self->LarkExportAuthor));
	lark_gstreamer_set_comment_tag(self->gstreamer, (gchar *)gtk_entry_get_text(self->LarkExportComment));
	
	g_signal_emit_by_name(G_OBJECT(self), "export-started");
	
	gtk_widget_destroy(GTK_WIDGET(self));
}

static void lark_export_dialog_close_button_clicked(GtkButton *button, LarkExportDialog *self)
{
	gtk_widget_destroy(GTK_WIDGET(self));
}

static void lark_export_dialog_file_name_changed(GtkEditable *editable, LarkExportDialog *self)
{
	GtkEntryBuffer *buffer;
	
	buffer = gtk_entry_get_buffer(self->LarkExportFile);
	if (buffer != NULL) {
		if (gtk_entry_buffer_get_length(buffer) > 0) {
			gtk_widget_set_sensitive(GTK_WIDGET(self->LarkExportExportButton), TRUE);
		} else {
			gtk_widget_set_sensitive(GTK_WIDGET(self->LarkExportExportButton), FALSE);
		}
	}
}

static void lark_export_dialog_format_changed(GtkComboBox *widget, LarkExportDialog *self)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	gint id;
	
	model = gtk_combo_box_get_model(widget);
	
	if (model != NULL) {
		if (gtk_combo_box_get_active_iter(widget, &iter)) {
			gtk_tree_model_get(model, &iter, LARK_EXPORT_DIALOG_FORMAT_ID_COLUMN, &id, -1);
			
			switch (id) {
				case LARK_GSTREAMER_TARGET_WAV:
					
					break;
				case LARK_GSTREAMER_TARGET_VORBIS:
					
					break;
				case LARK_GSTREAMER_TARGET_OPUS:
					
					break;
				case LARK_GSTREAMER_TARGET_FLAC:
					
					break;
				case LARK_GSTREAMER_TARGET_MP3:
					
					break;
				case LARK_GSTREAMER_TARGET_AAC:
					
					break;
				default:
					
					break;
			}
		}
	}
}

static void lark_export_dialog_limit_mode_changed(GtkComboBox *widget, LarkExportDialog *self)
{
	switch (gtk_combo_box_get_active(widget)) {
		case LARK_EXPORT_DIALOG_SPLIT_MODE_SIZE:
			gtk_adjustment_set_lower(self->LarkExportSplitLimitAdjustment, LARK_GSTREAMER_MIN_SIZE_LIMIT);
			gtk_adjustment_set_upper(self->LarkExportSplitLimitAdjustment, LARK_GSTREAMER_MAX_SIZE_LIMIT);
			gtk_adjustment_set_step_increment(self->LarkExportSplitLimitAdjustment, LARK_GSTREAMER_MIN_SIZE_LIMIT);
			gtk_adjustment_set_page_increment(self->LarkExportSplitLimitAdjustment, 100.0 * LARK_GSTREAMER_MIN_SIZE_LIMIT);
			gtk_adjustment_set_value(self->LarkExportSplitLimitAdjustment, (gdouble)lark_gstreamer_get_size_limit(self->gstreamer));
			gtk_widget_set_sensitive(GTK_WIDGET(self->LarkExportSplitLimit), TRUE);
			break;
		case LARK_EXPORT_DIALOG_SPLIT_MODE_TIME:
			gtk_adjustment_set_lower(self->LarkExportSplitLimitAdjustment, LARK_GSTREAMER_MIN_TIME_LIMIT);
			gtk_adjustment_set_upper(self->LarkExportSplitLimitAdjustment, LARK_GSTREAMER_MAX_TIME_LIMIT);
			gtk_adjustment_set_step_increment(self->LarkExportSplitLimitAdjustment, LARK_GSTREAMER_MIN_TIME_LIMIT);
			gtk_adjustment_set_page_increment(self->LarkExportSplitLimitAdjustment, 60.0 * LARK_GSTREAMER_MIN_TIME_LIMIT);
			gtk_adjustment_set_value(self->LarkExportSplitLimitAdjustment, (gdouble)lark_gstreamer_get_time_limit(self->gstreamer));
			gtk_widget_set_sensitive(GTK_WIDGET(self->LarkExportSplitLimit), TRUE);
			break;
		default:
			gtk_adjustment_set_lower(self->LarkExportSplitLimitAdjustment, 0.0);
			gtk_adjustment_set_upper(self->LarkExportSplitLimitAdjustment, 1.0);
			gtk_adjustment_set_step_increment(self->LarkExportSplitLimitAdjustment, 1.0);
			gtk_adjustment_set_page_increment(self->LarkExportSplitLimitAdjustment, 1.0);
			gtk_adjustment_set_value(self->LarkExportSplitLimitAdjustment, 0.0);
			gtk_widget_set_sensitive(GTK_WIDGET(self->LarkExportSplitLimit), FALSE);
			break;
	}
	
	g_signal_emit_by_name(G_OBJECT(self->LarkExportSplitLimitAdjustment), "value-changed", self);
}

static gchar *lark_export_dialog_limit_format_value(GtkScale *scale, gdouble value, LarkExportDialog *self)
{
	GString *res;
	guint t;
	
	switch (gtk_combo_box_get_active(GTK_COMBO_BOX(self->LarkExportSplitMode))) {
		case LARK_EXPORT_DIALOG_SPLIT_MODE_SIZE:
			res = g_string_new(NULL);
			g_string_append_printf(res, "%.0f Mb", value / (1000.0 * 1000.0));
			gtk_widget_set_sensitive(GTK_WIDGET(self->LarkExportSplitLimit), TRUE);
			break;
		case LARK_EXPORT_DIALOG_SPLIT_MODE_TIME:
			res = g_string_new(NULL);
			t = (guint)value / 3600;
			if (t > 0) {
				g_string_append_printf(res, "%u:", t);
			}
			t = ((guint)value % 3600) / 60;
			if (t < 10) {
				g_string_append_printf(res, "0%u:", t);
			} else {
				g_string_append_printf(res, "%u:", t);
			}
			t = (guint)value % 60;
			if (t < 10) {
				g_string_append_printf(res, "0%u", t);
			} else {
				g_string_append_printf(res, "%u", t);
			}
			break;
		default:
			res = g_string_new("");
			break;
	}
	
	return g_string_free(res, FALSE);
}

static void lark_export_dialog_format_set_sensetive(GtkCellLayout *cell_layout, GtkCellRenderer *cell, GtkTreeModel *tree_model, GtkTreeIter *iter, LarkExportDialog *self)
{
	gint id;
	
	g_return_if_fail(self != NULL);
	
	gtk_tree_model_get(tree_model, iter, LARK_EXPORT_DIALOG_FORMAT_ID_COLUMN, &id, -1);
		
	switch (id) {
		case LARK_GSTREAMER_TARGET_WAV:
			g_object_set(cell, "sensitive", TRUE, NULL);
			break;
		case LARK_GSTREAMER_TARGET_VORBIS:
			g_object_set(cell, "sensitive", lark_gstreamer_get_vorbis_available(self->gstreamer), NULL);
			break;
		case LARK_GSTREAMER_TARGET_OPUS:
			g_object_set(cell, "sensitive", lark_gstreamer_get_opus_available(self->gstreamer), NULL);
			break;
		case LARK_GSTREAMER_TARGET_FLAC:
			g_object_set(cell, "sensitive", lark_gstreamer_get_flac_available(self->gstreamer), NULL);
			break;
		case LARK_GSTREAMER_TARGET_MP3:
			g_object_set(cell, "sensitive", lark_gstreamer_get_mp3_available(self->gstreamer), NULL);
			break;
		case LARK_GSTREAMER_TARGET_AAC:
			g_object_set(cell, "sensitive", lark_gstreamer_get_aac_available(self->gstreamer), NULL);
			break;
		default:
			g_object_set(cell, "sensitive", FALSE, NULL);
			break;
	}
}

static void lark_export_dialog_get_property(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
{
	LarkExportDialog *self = (LarkExportDialog *)object;
	
	switch (prop_id) {
		case PROP_GSTREAMER:
			g_value_set_pointer(value, self->gstreamer);
			break;
		case PROP_DOCUMENT_LOADER:
			g_value_set_pointer(value, self->docloader);
			break;
		case PROP_EXPORT_SETTINGS:
			g_value_set_pointer(value, self->expsettings);
			break;
	}
}

static void lark_export_dialog_set_property(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
{
	LarkExportDialog *self = (LarkExportDialog *)object;
	
	switch (prop_id) {
		case PROP_GSTREAMER:
			self->gstreamer = g_value_get_pointer(value);
			break;
		case PROP_DOCUMENT_LOADER:
			self->docloader = g_value_get_pointer(value);
			break;
		case PROP_EXPORT_SETTINGS:
			self->expsettings = g_value_get_pointer(value);
			break;
	}
}

static void lark_export_dialog_constructed(GObject *object)
{
	LarkExportDialog *self = (LarkExportDialog *)object;
	GtkCellRenderer *renderer;
	gchar *metastr;
	
	/*Format combo box*/
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(self->LarkExportFormat), renderer, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(self->LarkExportFormat), renderer, "markup", LARK_EXPORT_DIALOG_FORMAT_NAME_COLUMN, NULL);
	g_object_set(G_OBJECT(renderer), "ellipsize", PANGO_ELLIPSIZE_END, "max-width-chars", 35, NULL);
	gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(self->LarkExportFormat), renderer, (GtkCellLayoutDataFunc)lark_export_dialog_format_set_sensetive, self, NULL);
	gtk_widget_realize(GTK_WIDGET(self->LarkExportFormat));
	
	/*Filename*/
	metastr = lark_document_loader_get_filename(self->docloader);
	if (metastr != NULL) {
		gtk_entry_set_text(GTK_ENTRY(self->LarkExportFile), metastr);
		g_free(metastr);
	} else {
		gtk_entry_set_text(GTK_ENTRY(self->LarkExportFile), "audiobook");
	}
	/*Title*/
	metastr = lark_document_loader_get_title(self->docloader);
	if (metastr != NULL) {
		gtk_entry_set_text(GTK_ENTRY(self->LarkExportTitle), metastr);
	}
	/*Author*/
	metastr = lark_document_loader_get_author(self->docloader);
	if (metastr != NULL) {
		gtk_entry_set_text(GTK_ENTRY(self->LarkExportAuthor), metastr);
	}
	/*Comment*/
	metastr = lark_document_loader_get_comment(self->docloader);
	if (metastr != NULL) {
		gtk_entry_set_text(GTK_ENTRY(self->LarkExportComment), metastr);
	}
	
	/*Set initial values*/
	gtk_combo_box_set_active(self->LarkExportFormat, lark_gstreamer_get_target(self->gstreamer) - 1);
	gtk_adjustment_set_value(self->LarkExportQualityAdjustment, (gdouble)lark_gstreamer_get_quality(self->gstreamer));
	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(self->LarkExportFolder), lark_gstreamer_get_path(self->gstreamer));
	
	switch (lark_gstreamer_get_limit_mode(self->gstreamer)) {
		case LARK_GSTREAMER_LIMIT_MODE_SIZE:
			gtk_combo_box_set_active(GTK_COMBO_BOX(self->LarkExportSplitMode), LARK_EXPORT_DIALOG_SPLIT_MODE_SIZE);
			break;
		case LARK_GSTREAMER_LIMIT_MODE_TIME:
			gtk_combo_box_set_active(GTK_COMBO_BOX(self->LarkExportSplitMode), LARK_EXPORT_DIALOG_SPLIT_MODE_TIME);
			break;
		default:
			gtk_combo_box_set_active(GTK_COMBO_BOX(self->LarkExportSplitMode), LARK_EXPORT_DIALOG_SPLIT_MODE_DISABLED);
			break;
		
	}
}

static void lark_export_dialog_class_init(LarkExportDialogClass *klass)
{
	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(klass);
	GObjectClass *object_class = G_OBJECT_CLASS(klass);
	
	object_class->get_property = lark_export_dialog_get_property;
	object_class->set_property = lark_export_dialog_set_property;
	object_class->constructed = lark_export_dialog_constructed;
	
	properties[PROP_GSTREAMER] =
		g_param_spec_pointer("gstreamer",
							"GStreamer object",
							"Pointer to GStreamer object",
							G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
	
	properties[PROP_DOCUMENT_LOADER] =
		g_param_spec_pointer("docloader",
							"Document loader object",
							"Pointer to document loader object",
							G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
							
	properties[PROP_EXPORT_SETTINGS] =
		g_param_spec_pointer("export-settings",
							"Export settings object",
							"Pointer to export settings object",
							G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);
								
	g_object_class_install_properties(object_class, LAST_PROP, properties);
	
	signals[SIGNAL_EXPORT_STARTED] =
		g_signal_new("export-started",
						G_TYPE_FROM_CLASS (klass),
						G_SIGNAL_RUN_FIRST,
						0,
						NULL,
						NULL,
						g_cclosure_marshal_VOID__VOID,
						G_TYPE_NONE,
						0,
						G_TYPE_NONE);
	
	gtk_widget_class_set_template_from_resource(widget_class, "/ru/linuxonly/lark/ui/export-dialog.ui");
	
	gtk_widget_class_bind_template_child(widget_class, LarkExportDialog, LarkExportExportButton);
	gtk_widget_class_bind_template_child(widget_class, LarkExportDialog, LarkExportFile);
	gtk_widget_class_bind_template_child(widget_class, LarkExportDialog, LarkExportFolder);
	gtk_widget_class_bind_template_child(widget_class, LarkExportDialog, LarkExportFormat);
	gtk_widget_class_bind_template_child(widget_class, LarkExportDialog, LarkExportQuality);
	gtk_widget_class_bind_template_child(widget_class, LarkExportDialog, LarkExportQualityAdjustment);
	
	gtk_widget_class_bind_template_child(widget_class, LarkExportDialog, LarkExportSplitMode);
	gtk_widget_class_bind_template_child(widget_class, LarkExportDialog, LarkExportSplitLimit);
	gtk_widget_class_bind_template_child(widget_class, LarkExportDialog, LarkExportSplitLimitAdjustment);
	
	gtk_widget_class_bind_template_child(widget_class, LarkExportDialog, LarkExportTitle);
	gtk_widget_class_bind_template_child(widget_class, LarkExportDialog, LarkExportAuthor);
	gtk_widget_class_bind_template_child(widget_class, LarkExportDialog, LarkExportComment);
	
	gtk_widget_class_bind_template_callback(widget_class, lark_export_dialog_export_button_clicked);
	gtk_widget_class_bind_template_callback(widget_class, lark_export_dialog_close_button_clicked);
	
	gtk_widget_class_bind_template_callback(widget_class, lark_export_dialog_file_name_changed);
	gtk_widget_class_bind_template_callback(widget_class, lark_export_dialog_format_changed);
	
	gtk_widget_class_bind_template_callback(widget_class, lark_export_dialog_limit_mode_changed);
	gtk_widget_class_bind_template_callback(widget_class, lark_export_dialog_limit_format_value);
}

static void lark_export_dialog_init(LarkExportDialog *self)
{
	gtk_widget_init_template(GTK_WIDGET(self));
}

GtkWidget *lark_export_dialog_new(GtkWindow *parent, LarkGstreamer *gstreamer, LarkDocumentLoader *docloader, GSettings *expsettings)
{
	return g_object_new(lark_export_dialog_get_type(), "transient-for", parent, "use-header-bar", TRUE, "gstreamer", gstreamer, "docloader", docloader, "export-settings", expsettings, NULL);
}
