/*
 *      lark-tts-engine.h
 *
 *      Copyright 2016-2018 Alex <alex@linuxonly.ru>
 *
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef LARK_TTS_ENGINE_H
#define LARK_TTS_ENGINE_H

#include <glib.h>
#include <glib-object.h>
#include <glib/gi18n.h>

#include "lark-gstreamer.h"

G_BEGIN_DECLS

enum _LarkTtsEngineVoiceGender {
	LARK_TTS_ENGINE_VOICE_GENDER_UNKNOWN = 0,
	LARK_TTS_ENGINE_VOICE_GENDER_MALE,
	LARK_TTS_ENGINE_VOICE_GENDER_FEMALE
};

enum _LarkTtsCapitalsSymbolsIndication {
	LARK_TTS_CAPITALS_SYMBOLS_INDICATION_DEFAULT = 0,
	LARK_TTS_CAPITALS_SYMBOLS_INDICATION_PITCH,
	LARK_TTS_CAPITALS_SYMBOLS_INDICATION_VOLUME,
	LARK_TTS_CAPITALS_SYMBOLS_INDICATION_OFF
};

enum _LarkTtsPunctuationSymbolsIndication {
	LARK_TTS_PUNCTUATION_SYMBOLS_INDICATION_DEFAULT = 0,
	LARK_TTS_PUNCTUATION_SYMBOLS_INDICATION_ON,
	LARK_TTS_PUNCTUATION_SYMBOLS_INDICATION_OFF
};

struct _LarkTtsEngineVoice {
	gchar *name;
	GPtrArray *languages;
	GArray *genders;
};

typedef struct _LarkTtsEngineVoice LarkTtsEngineVoice;

#define LARK_TYPE_TTS_ENGINE (lark_tts_engine_get_type ())

G_DECLARE_DERIVABLE_TYPE (LarkTtsEngine, lark_tts_engine, LARK, TTS_ENGINE, GObject)

struct _LarkTtsEngineClass {
	GObjectClass parent_class;
};

LarkTtsEngine *lark_tts_engine_new(void);
GSList *lark_tts_engine_get_voices(LarkTtsEngine *engine);
gint lark_tts_engine_get_format(LarkTtsEngine *engine);
gint lark_tts_engine_get_samplerate(LarkTtsEngine *engine);
gint lark_tts_engine_get_channels(LarkTtsEngine *engine);
void lark_tts_engine_set_voice_profile(LarkTtsEngine *engine, gchar *voiceprofile);
gchar *lark_tts_engine_get_voice_profile(LarkTtsEngine *engine);
void lark_tts_engine_set_rate(LarkTtsEngine *engine, gdouble rate);
gdouble lark_tts_engine_get_rate(LarkTtsEngine *engine);
void lark_tts_engine_set_pitch(LarkTtsEngine *engine, gdouble pitch);
gdouble lark_tts_engine_get_pitch(LarkTtsEngine *engine);
void lark_tts_engine_set_volume(LarkTtsEngine *engine, gdouble volume);
gdouble lark_tts_engine_get_volume(LarkTtsEngine *engine);
void lark_tts_engine_set_capitals_indication(LarkTtsEngine *engine, gint capitalsind);
gint lark_tts_engine_get_capitals_indication(LarkTtsEngine *engine);
void lark_tts_engine_set_punctuation_indication(LarkTtsEngine *engine, gint punctuationind);
gint lark_tts_engine_get_punctuation_indication(LarkTtsEngine *engine);
gchar *lark_tts_engine_format_voice_description(LarkTtsEngineVoice *voice);

G_END_DECLS

#endif /* LARK_TTS_ENGINE_H */
